
import xarray as xr
import metacsv
import datafs
import os


class DataCache(object):

    _api = None
    _data = {}

    @classmethod
    def retrieve(cls, archive_name, api=None):

        if archive_name in cls._data:
            return cls._data[archive_name]

        if api is None:

            if cls._api is None:
                cls._api = datafs.get_api()

            api = cls._api

        archive = api.get_archive(archive_name)

        if archive_name.endswith('.csv'):
            with archive.open() as f:
                data = metacsv.read_csv(f)

        elif os.path.splitext(archive_name)[-1][:3] == '.nc':
            with archive.get_local_path() as fp:
                data = xr.open_dataset(fp).load()

        else:
            raise ValueError('file type not recognized: "{}"'.format(
                archive_name))

        cls._data[archive_name] = data

        return data



"""System for handling the parallel processing on the file system.

The use case is when we design a job to be able to be run multiple
times on the same server, where the jobs each work on distinct
directories.  Each job "claims" a directory when it begins working on
it and releases the directory when it is done.

We do this for the forecast generation, aggregation, result checking,
and cleanup jobs.

The system should handle situations where the process crashes before
it has released a directory; where we want to kill all processes early
and release their directories; where we want to track down logs for
each output; and where jobs of one kind either can or cannot be run
while jobs of another kind are active.

Usage:

 - Each process instantiates StatusManager, which begins a new log file.

 - When the process begins work in a directory, it calls "claim",
   which attempts to lock the directory and creates a job-specific
   status file, while represents that lock.

 - When the process complete it's work, it calls "release", which
   deletes the job-specific status file, but addes a new status line
   to the cross-job status file.
"""

import sys, os, itertools, time, signal

class StatusManager(object):
    def __init__(self, jobname, jobtitle, logdir, timeout, exclusive_jobnames=None):
        """
        Create a log file to capture all output, and set up to claim directories.

        Args:
            jobname: A short name for the job, used in the claim filename (e.g., aggregate)
            jobtitle: A short descriptive title for the job (e.g., "generate.aggregate configs/mortality-montecarlo.yml")
            logdir: The directory for log files.
            timeout: Seconds after which a job has started on a directory that we assume it has crashed and allow the directory to be re-claimed.
            exclusive_jobnames: Other job names which cannot be running in the same directory.  Note that `timeout` should be at least as long as the timeouts associated with these.
        """

        self.jobname = jobname
        self.jobtitle = jobtitle
        self.timeout = timeout
        self.exclusive_jobnames = exclusive_jobnames if exclusive_jobnames is not None else []

        # Decide on the name of the log file
        if not os.path.exists(logdir):
            os.makedirs(logdir)
        for ii in itertools.count():
            logpath = os.path.join(logdir, "%s-%d.log" % (jobname, ii))
            if not os.path.exists(logpath):
                self.logpath = logpath
                break

        # Grab all std out
        self.sys_stdout = sys.stdout
        sys.stdout = DoubleLogger(self.logpath)

    def __del__(self):
        """Usually not used; allow std output to go back to its previous stream."""
        sys.stdout.close()
        sys.stdout = self.sys_stdout

    def claim(self, dirpath):
        """Claim a directory."""
        if not os.path.exists(dirpath):
            os.makedirs(os.path.abspath(dirpath))
        elif self.is_claimed(dirpath):
            return False

        status_path = StatusManager.claiming_filepath(dirpath, self.jobname)
        with open(status_path, 'w') as fp:
            fp.write("%d %s: %s\n" % (os.getpid(), self.jobtitle, self.logpath))

        return True

    def update(self, dirpath, status):
        """Provide additional status information."""
        status_path = StatusManager.claiming_filepath(dirpath, self.jobname)
        with open(status_path, 'a') as fp:
            fp.write(status + '\n')

    def release(self, dirpath, status):
        """Release the claim on this directory."""
        os.remove(StatusManager.claiming_filepath(dirpath, self.jobname))

        with open(StatusManager.globalstatus_filepath(dirpath), 'a') as fp:
            fp.write("%s %s: %s\n" % (time.asctime(), self.jobtitle, status))

    def is_claimed(self, dirname):
        """Check if a directory has claims from any of our jobs."""
        if not os.path.exists(dirname):
            return False

        for jobname in [self.jobname] + self.exclusive_jobnames:
            filepath = StatusManager.claiming_filepath(dirname, jobname)
            if os.path.exists(filepath):
                if time.time() - os.path.getmtime(filepath) < self.timeout:
                    return True
                # If after the timeout, leave it there but ignore it.

        return False

    @staticmethod
    def globalstatus_filepath(dirpath):
        """The path to the global status for the directory."""
        return StatusManager.claiming_filepath(dirpath, 'global')

    @staticmethod
    def claiming_filepath(dirpath, jobname):
        """Return the path to the status file used for claiming a directory."""
        return os.path.join(dirpath, "status-%s.txt" % jobname)

    @staticmethod
    def kill_active(dirpath, jobname):
        """Kill any job claiming this directory."""
        filepath = StatusManager.claiming_filepath(dirpath, jobname)

        if not os.path.exists(filepath):
            return

        with open(filepath, 'r') as fp:
            status = fp.read()
            pid = int(status.split()[0])
            os.kill(pid, signal.SIGTERM)

        os.remove(filepath)

class DoubleLogger(object):
    """From http://stackoverflow.com/questions/14906764/how-to-redirect-stdout-to-both-file-and-console-with-scripting"""
    def __init__(self, logpath):
        self.terminal = sys.stdout
        self.log = open(logpath, "a")

    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)

    def close(self):
        self.log.close()

    def flush(self):
        #this flush method is needed for python 3 compatibility.
        #this handles the flush command by doing nothing.
        #you might want to specify some extra behavior here.
        pass



"""
Utilities for path handling

Provides server-specific paths, configured in a server configuration file.
"""

import sys
import os
import yaml

default_server_config_path = "../server.yml"
server_config = None


# Path-handling functions

def sharedpath(subpath):
    """Return a subpath of the configured shareddir."""
    if server_config is None:
        msg = "Cannot find configuration file at {}".format(
            default_server_config_path)

        assert os.path.exists(default_server_config_path), msg

        use_config(get_file_config(default_server_config_path))

    return os.path.join(server_config['shareddir'], subpath)

def configpath(path):
    """Return an configured absolute path.  If the path is absolute, it
    will be left alone; otherwise, it is assumed to be a subpath of the
    configured `shareddir`."""

    if path[0] == '/':
        return path

    return sharedpath(path)


# Configuration-file handling functions

def use_config(config):
    """Use the given configuration for path functions."""
    global server_config

    # Add values from server_config only if not in config
    if server_config is not None:
        for key in server_config:
            if key not in config:
                config[key] = server_config[key]

    server_config = config

def get_file_config(filepath):
    """Load a configuration file from a given path."""

    with open(filepath, 'r') as fp:
        config = yaml.load(fp)
        return config

def get_argv_config(index=1):
    """
    Load a configuration file specified as the `index` argv argument.

    In the future, this should also load specific configurable options from the
    command-line.
    """

    with open(sys.argv[index], 'r') as fp:
        config = yaml.load(fp)
        return config

def get_allargv_config():
    """
    Load a configuration from the command line, merging all arguments into a single configuration dictionary.

    Handles the following kinds of command-line arguments:
    - `*.yml`: a full configuration YaML file, merged into the result dictionary
    - `--config=VALUE`: a full configuration YaML file, as above
    - `--KEY=VALUE`: a single configuration value to be set
    - anything else: sets an entry in the config file for that anything to have a value of True

    Later command-line arguments always overide earlier ones.
    """
    config = {}
    for arg in sys.argv:
        if arg[-4:] == '.yml':
            config.update(get_file_config(arg))
            continue

        # Add --key=value arguments one at a time
        if arg[0:2] == '--':
            if '=' in arg:
                chunks = arg[2:].split('=')
                if chunks[0] == 'config':
                    config = get_file_config(chunks[1])
                else:
                    config[chunks[0]] = yaml.safe_load(chunks[1])
            else:
                config[arg[2:]] = True
            continue

        config[arg] = True

    return config

if __name__ == '__main__':
    print(configpath('testing'))




from __future__ import absolute_import

from impactlab_tools.utils.weighting import weighted_quantile_xr
from impactlab_tools.utils.cache import DataCache


def acp_quantiles(
        data,
        rcp,
        quantiles=[0.05, 0.167, 0.5, 0.833, 0.95],
        values_sorted=False,
        dim='model',
        api=None):
    """
    Compute quantiles of an xarray distribution using ACP weights

    .. NOTE ::

        quantiles should be in [0, 1]!

    Parameters
    ----------

    data : DataArray

        xarray.DataArray with data

    rcp : str

        RCP weights/models to use ('rcp26', 'rcp45', 'rcp60', 'rcp85')

    quantiles : array-like

        quantiles of distribution to return

    values_sorted : bool

        if True, then will avoid sorting of initial array

    dim : str

        dimension along which to retrieve quantiles. The indices of this
        dimension should be valid ACP climate models.

    api : object

        DataFS API object to use in data retrieval (optional, uses default
        profile if not provided)


    Returns
    -------

    xarray.DataArray

        computed quantiles from weighted distribution

    See also
    --------

    * :py:func:`.utils.weighting.weighted_quantile_xr`
    * :py:func:`.utils.weighting.weighted_quantile`
    * :py:func:`.utils.weighting.weighted_quantile_1d`
    * :py:func:`numpy.percentile`
    * :py:meth:`pandas.DataFrame.quantile`
    * :py:meth:`pandas.Series.quantile`

    """

    mw = DataCache.retrieve('ACP_climate_gcm-modelweights.csv').xs(
        rcp, level='rcp')['weight_corrected']

    return weighted_quantile_xr(
        data=data,
        quantiles=quantiles,
        sample_weight=mw,
        values_sorted=False,
        dim=dim)



import shutil
from impactlab_tools.utils import paralog

def test_claiming():
    statman1 = paralog.StatusManager('test', 'Testing process', 'testing-paralog', 60*60)
    print statman1.logpath
    assert statman1.claim("testing-paralog"), "Cannot claim directory!"

    statman2 = paralog.StatusManager('test', 'Testing process', 'testing-paralog', 60*60)
    print statman2.logpath
    assert not statman2.claim("testing-paralog"), "Accidentally claimed directory!"

    statman1.update("testing-paralog", "New status.")
    statman1.release("testing-paralog", "First pass complete.")

    assert statman2.claim("testing-paralog"), "Cannot claim directory afterwards!"

    statman2.release("testing-paralog", "Second pass complete.")

    logpath1 = statman1.logpath
    logpath2 = statman2.logpath

    del statman2 # need to delete in opposite order for our test
    del statman1

    print logpath1
    print logpath2

    with open(paralog.StatusManager.globalstatus_filepath("testing-paralog"), 'r') as fp:
        print fp.read()

    shutil.rmtree('testing-paralog')



import xarray as xr
import numpy as np
import scipy.stats

from pandas import CategoricalIndex


def binned_statistic_1d(da, dim, bins=10, statistic='count', value_range=None):
    '''
    Bin a data array by values and summarize along a dimension

    Parameters
    ----------

    da : xr.DataArray
        DataArray to be binned

    dim : str
        Dimension along which to summarize the binned values

    statistic : string or callable, optional
        The statistic to compute (default is 'count'). The following statistics
        are available:

        * 'mean' : compute the mean of values for points within each bin. Empty
            bins will be represented by NaN.

        * 'median' : compute the median of values for points within each bin.
            Empty bins will be represented by NaN.

        * 'count' : compute the count of points within each bin. This is
            identical to an unweighted histogram. values array is not
            referenced.

        * 'sum' : compute the sum of values for points within each bin. This is
            identical to a weighted histogram.

        * 'min' : compute the minimum of values for points within each bin.
            Empty bins will be represented by NaN.

        * 'max' : compute the maximum of values for point within each bin.
            Empty bins will be represented by NaN.

        * function : a user-defined function which takes a 1D array of values,
            and outputs a single numerical statistic. This function will be
            called on the values in each bin. Empty bins will be represented by
            function([]), or NaN if this returns an error.

    bins : int or sequence of scalars, optional
        If bins is an int, it defines the number of equal-width bins in the
        given range (10 by default). If bins is a sequence, it defines the bin
        edges, including the rightmost edge, allowing for non-uniform bin
        widths. Values in x that are smaller than lowest bin edge are assigned
        to bin number 0, values beyond the highest bin are assigned to
        bins[-1]. If the bin edges are specified, the number of bins will be,
        (nx = len(bins)-1).


    value_range : (float, float) or [(float, float)], optional
        The lower and upper range of the bins. If not provided, value_range is
        simply (x.min(), x.max()). Values outside the range are ignored.

    Returns
    -------

    binned : xr.DataArray
        A data array with bins along the summary dimension

    Examples
    --------

    .. code-block:: python

        >>> da = xr.DataArray(
        ...     np.arange(16).reshape(4,4),
        ...     dims=('a', 'b'),
        ...     coords={'a': list('abcd'), 'b': list('wxyz')})
        ...
        >>> da
        <xarray.DataArray (a: 4, b: 4)>
        array([[ 0,  1,  2,  3],
               [ 4,  5,  6,  7],
               [ 8,  9, 10, 11],
               [12, 13, 14, 15]])
        Coordinates:
          * a        (a) |S1 'a' 'b' 'c' 'd'
          * b        (b) |S1 'w' 'x' 'y' 'z'

        >>> binned_statistic_1d(da, 'b', [0, 2, 5, 20])
        <xarray.DataArray (a: 4, groups: 3)>
        array([[ 2.,  2.,  0.],
               [ 0.,  1.,  3.],
               [ 0.,  0.,  4.],
               [ 0.,  0.,  4.]])
        Coordinates:
          * a        (a) |S1 'a' 'b' 'c' 'd'
          * groups   (groups) object '(0, 2]' '(2, 5]' '(5, 20]'

        >>> binned_statistic_1d(da, 'a', statistic='sum')
        <xarray.DataArray (groups: 10, b: 4)>
        array([[  0.,   1.,   2.,   3.],
               [  0.,   0.,   0.,   0.],
               [  0.,   0.,   0.,   0.],
               [  4.,   5.,   6.,   7.],
               [  0.,   0.,   0.,   0.],
               [  0.,   0.,   0.,   0.],
               [  8.,   9.,  10.,  11.],
               [  0.,   0.,   0.,   0.],
               [  0.,   0.,   0.,   0.],
               [ 12.,  13.,  14.,  15.]])
        Coordinates:
          * b        (b) |S1 'w' 'x' 'y' 'z'
          * groups   (groups) object '(0.0, 1.5]' '(1.5, 3.0]' '(3.0, 4.5]' ...
    '''

    # apply binned_statistic along dim
    bnd = np.apply_along_axis(
        lambda x, **kwds: scipy.stats.binned_statistic(x, x, **kwds)[0],
        da.get_axis_num(dim),
        da.values,
        bins=bins,
        statistic=statistic,
        range=value_range)

    if isinstance(bins, int):
        if value_range is None:
            value_range = float(da.min()), float(da.max())
        bins = np.linspace(value_range[0], value_range[1], bins+1)

    # build index for new array
    bindex = CategoricalIndex(
        ['({}, {}]'.format(bins[i-1], bins[i]) for i in xrange(1, len(bins))],
        ordered=True)

    da = xr.DataArray(
        bnd,
        dims=tuple([
            d if d != dim else 'groups' for d in da.dims]),
        coords={
            d if d != dim else 'groups': da.coords[d] if d != dim else bindex
            for d in da.dims})
    return da




import subprocess as sp


def check_version(input_list, check_git=False):
    """
    Returns version information given a list of module dependencies

    Parameters
    ----------

    input_list: list
        list of strings, all module names

    check_git: bool
        True if the caller also wants to check the git hash of a repo
        (input_list contains its name) that's under the user's home dir.

    Returns
    -------

    dict:
        A dictionary of the modules: keys are the module names, each
        key has value of another dictionary, containing:

            - "source": how is the module installed ("pip", "local",
              "git", or None):

                - source is "pip" if it's an open-sourced python
                  package installed through pip.
                - source is "pip-local" if it's a self-made tool
                  installed through pip.
                - source is "git" if it's a git managed repo of
                  scripts, not installed through pip.
                - source is None if the module cannot be found.

            - "version": If it's an open source module (source: pip),
              this is the version numbers of it.
            - "git_hash": If it's a local module (source: local, or
              git).

    Example
    -------

    >>> input_list = [
    ...    "scipy", "numpy", "Cheetah", "computer",
    ...    "impact-calculations", "metacsv"]
    ...
    >>> check_version(input_list, check_git=True) # doctest: +SKIP
    {
        "scipy": {"source": "pip", "version": "0.19"},
        "numpy": {"source": "pip", "version": "1.12.1"},
        "Cheetah": {"source": "pip", "version": "2.4.4"},
        "computer": {
            "source": "git",
            "git_hash": "662870e0fa914b4fa958e78ebe02b858c31fe41d"},
        "impact-calculations": {
            "source": "git",
            "git_hash": "e7c1b53b1d9e6571c0555a560c919f9645693b45"},
        "metacsv": {"source": "pip", "version": "0.0.9"}
    }

    """
    # Read in all packages in pips
    pips = sp.check_output("pip freeze", shell=True).split("\n")[:-1]
    modules = {}
    for mod in pips:
        if ".git@" in mod:
            # Assume the format of a local repo in pip freeze looks
            # like this:
            #
            #     "-e git+https://github.com/<UserName>/
            #         <RepoName>.git@<Git_Hash>#egg=computer-master"
            #
            # We are interested in the <Git_Hash> part, saving in
            # the modules dictionary with <RepoName> as the key.
            ind = mod.index(".git@")
            end = mod.index("#")
            modules[mod[:ind].split("/")[-1]] = mod[ind+5:end]
        else:
            ind = mod.index("=")
            modules[mod[:ind]] = mod[ind:].rstrip("\n").lstrip("=")

    # Read in all repos under my home directory
    repos = sp.check_output(
        "find ~/ -name '.git'", shell=True).split(".git\n")[:-1]

    git = {}
    for repo in repos:
        name = repo.split("/")[-2]
        git[name] = sp.check_output(
            "cd " + repo + " && git log --format='%H' -n 1",
            shell=True).rstrip("\n")

    # Iterate through the input_list and find if the target is in
    # either the modules or the git dictionary, or not at all.
    rtDict = {}
    for tgt in input_list:
        info = {}
        if tgt in modules:
            if "." in modules[tgt]:
                info["source"] = "pip"
                info["version"] = modules[tgt]
            else:
                info["source"] = "pip-local"
                info["git_hash"] = modules[tgt]
        elif tgt in git:
                info["source"] = "git"
                info["git_hash"] = git[tgt]
        else:
            info["source"] = None
        rtDict[tgt] = info

    return rtDict



# -*- coding: utf-8 -*-
'''
'''

from __future__ import absolute_import

__author__ = """Climate Impact Lab"""
__version__ = '0.0.6'

_module_imports = (
)

__all__ = list(map(lambda x: x.__name__, _module_imports))




from __future__ import absolute_import

import pytest

import xarray as xr
import numpy as np
import pandas as pd

from impactlab_tools.utils import weighting


@pytest.yield_fixture
def increasing_array():

    yield xr.DataArray(
        np.sum(np.mgrid[0:5, 0:5, 0:5], axis=0),
        dims=('x', 'y', 'z'),
        coords={c: [c+str(i) for i in range(5)] for c in ['x', 'y', 'z']})


@pytest.yield_fixture
def random_array():

    yield xr.DataArray(
        np.random.random((5, 5, 5)),
        dims=('x', 'y', 'z'),
        coords={c: [c+str(i) for i in range(5)] for c in ['x', 'y', 'z']})


def test_unweighted_median(random_array):
    '''
    Asserts weighted_quantile_xr correctly returns the median
    '''

    for dim in random_array.dims:
        weighted = weighting.weighted_quantile_xr(
            random_array, [0.5], np.ones((5)), dim=dim)

        median = random_array.median(dim=dim)

        assert (weighted == median).all()


def test_first_value(random_array):
    '''
    Asserts weighted_quantile_xr correctly returns the first value
    '''

    for dim in random_array.dims:
        weighted = weighting.weighted_quantile_xr(
            random_array, [0.5], [1, 0, 0, 0, 0], dim=dim)

        first = random_array.isel(**{dim: 0})

        assert (weighted == first).all()


def test_manual_weighting(increasing_array):
    '''
    Asserts weighted_quantile_xr matches a manually weighted mean
    '''

    for dim in increasing_array.dims:
        weighted = weighting.weighted_quantile_xr(
            increasing_array, [0.5], [0.25, 0.5, 0, 0, 0.25], dim=dim)

        manual = increasing_array.isel(**{dim: [0, 1, 1, 4]}).median(dim=dim)

        assert (weighted == manual).all()


def test_unsorted_weights_index(increasing_array):
    '''
    Asserts weighted_quantile_xr weights correctly with unsorted weights
    '''

    for dim in increasing_array.dims:

        index = pd.Series(
            [0.25, 0.5, 0.25, 0, 0],
            index=pd.Index([dim+str(i) for i in [2, 4, 1, 3, 0]]))

        weighted = weighting.weighted_quantile_xr(
            increasing_array, [0.125, 0.5], index, dim=dim)

        manual = increasing_array.isel_points(
            dim=pd.Index([0.125, 0.5], name='quantile'),
            **{dim: [1, 3]})

        assert (weighted == manual).all()




import datafs
from impactlab_tools.utils.cache import DataCache


def population_weighted_mean(
        ds,
        level='state',
        dim='fips',
        year=2012,
        api=None,
        pop=None):
    '''
    Find the population-weighted mean of a county-level xarray DataArray

    Parameters
    ----------
    ds : array
        :py:class:`~xarray.DataArray` to be aggregated. May contain any number
        of dimensions >= 1.

    level : str (optional)
        Level of resolution to aggregate to. May be one of ``'fips'``,
        ``'state'``, ``'state_names'``, ``'state_abbrevs'``, ``'census'``,
        or ``'national'`` (default ``'state'``)

    dim : str (optional)
        dimension to aggregate along (default ``'fips'``)

    year : int (optional)
        population year (or column in the ``pop`` dataset) to use for the
        weights. If not provided, 2012 population is used.

    api : DataAPI (optional)
        :py:class:`datafs.DataAPI` object to use. If not provided, creates a
        new ``DataAPI`` object

    pop : array (optional)
        :py:class:`~xarray.DataArray` to use for weights. If not provided,
        US Census Bureau 2014 vintage CO-EST2014-alldata.csv estimates from
        the ACP are used

    Returns
    -------
    mean : array
        weighted average aggregated :py:class:`~xarray.DataArray`

    '''

    if pop is None and api is None:
        api = datafs.get_api()

    if pop is None:
        pop = _prep_pop_data(api)

        if dim != 'fips':
            pop = pop.rename({'fips': dim})

    return (
        ((ds * pop[str(year)]).groupby(level).sum(dim=dim)) /
        ((pop[str(year)]).groupby(level).sum(dim=dim)))


def _prep_pop_data(api):

    pop_arch = (
        'ACP/integration/socioeconomics/population/' +
        'census/county_census_pop.nc')

    return DataCache.retrieve(pop_arch, api=api)




import pytest


def test_top_level_imports():
    import impactlab_tools

    assert isinstance(impactlab_tools, type(pytest))


def test_acp_imports():
    from impactlab_tools.acp import dist, aggregate
    from impactlab_tools.utils import cache, weighting

    assert isinstance(dist, type(pytest))
    assert isinstance(aggregate, type(pytest))
    assert isinstance(cache, type(pytest))
    assert isinstance(weighting, type(pytest))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements_install = [
    'datafs>=0.6.9'
    ]

requirements_test = [
    'Sphinx>=1.4.1',
    'sphinx_rtd_theme>=0.1.0',
    'pip>=8.0',
    'wheel>=0.27',
    'flake8>=2.0',
    'tox>=2.3.0',
    'coverage>=4.0',
    'pytest>=3.0',
    'pytest_cov>=2.0',
    'pytest-runner>=2.5',
    'coveralls>=1.0'
    ]

requirements_conda = [
    'numpy>=1.7',
    'pandas>=0.15',
    'netCDF4>=1.1',
    'xarray>=0.8',
    'metacsv>=0.0.16'
]

extras = {
    'test': requirements_test,
    'conda': requirements_conda
}


setup(
    name='impactlab-tools',
    version='0.0.6',
    description="Python tools for Climate Impact Lab developers",
    long_description=readme,
    author="Climate Impact Lab",
    url='https://github.com/ClimateImpactLab/impactlab-tools',
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    package_dir={'impactlab_tools':
                 'impactlab_tools'},
    include_package_data=True,
    install_requires=requirements_install,
    license="MIT license",
    zip_safe=False,
    keywords='impactlab-tools',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7'
    ],
    test_suite='tests',
    setup_requires=['pytest-runner'],
    tests_require=requirements_test,
    extras_require=extras
)



import numpy as np
import pandas as pd
import xarray as xr


# Computation helpers

def weighted_quantile_xr(
        data,
        quantiles,
        sample_weight,
        dim,
        values_sorted=False):
    """
    Compute quantiles of a weighted distribution

    similar to :py:func:`weighted_quantile` operates on a named dimension of an
    :py:class:`xarray.DataArray`.

    .. NOTE ::

        quantiles should be in [0, 1]!

    Parameters
    ----------

    data : DataArray

        xarray.DataArray with data

    quantiles : array-like

        quantiles of distribution to return

    sample_weight : numpy.array

        weights array-like of the same length as `array`

    values_sorted : bool

        if True, then will avoid sorting of initial array

    dim : str

        Dimension along which to weight the data

    Returns
    -------

    xarray.DataArray

        computed quantiles from weighted distribution

    """

    axis = data.get_axis_num(dim)
    dims = list(data.dims[:axis]) + ['quantile'] + list(data.dims[axis+1:])
    coords = {
        coord: data.coords[coord] for coord in data.dims if coord in dims}
    coords.update({'quantile': quantiles})

    if isinstance(sample_weight, (pd.Series, xr.DataArray)):
        weights = sample_weight.loc[data.coords[dim].values].values
    else:
        weights = sample_weight

    data_dist = weighted_quantile(
        data.values,
        quantiles,
        weights,
        values_sorted=values_sorted,
        axis=axis)

    data_dist = xr.DataArray(data_dist, dims=dims, coords=coords)

    return data_dist


def weighted_quantile(
        values,
        quantiles,
        sample_weight=None,
        values_sorted=False,
        old_style=False,
        axis=None):
    """
    Compute quantiles of a weighted distribution

    similar to :py:func:`weighted_quantile_1d` but supports weighting along any
    (numbered) dimension

    .. NOTE ::

        quantiles should be in [0, 1]!

    Parameters
    ----------

    values : numpy.array

        numpy.array with data

    quantiles : array-like

        quantiles of distribution to return

    sample_weight : numpy.array

        weights array-like of the same length as `array`

    values_sorted : bool

        if True, then will avoid sorting of initial array

    old_style : bool

        if True, will correct output to be consistent with numpy.percentile.

    Returns
    -------

    numpy.array

        computed quantiles from weighted distribution

    """
    if len(values.shape) > 1 and axis is not None:

        return np.apply_along_axis(
            weighted_quantile_1d,
            axis,
            values,
            quantiles,
            sample_weight,
            values_sorted,
            old_style)

    else:

        return weighted_quantile_1d(
            values,
            quantiles,
            sample_weight,
            values_sorted,
            old_style)


def weighted_quantile_1d(
        values,
        quantiles,
        sample_weight=None,
        values_sorted=False,
        old_style=False):
    """
    Very close to numpy.percentile, but supports weights

    Thanks to Alleo!
    http://stackoverflow.com/questions/21844024/weighted-percentile-using-numpy/29677616#29677616

    .. NOTE ::

        quantiles should be in [0, 1]!

    Parameters
    ----------

    values : numpy.array

        numpy.array with data

    quantiles : array-like

        quantiles of distribution to return

    sample_weight : numpy.array

        weights array-like of the same length as `array`

    values_sorted : bool

        if True, then will avoid sorting of initial array

    old_style : bool

        if True, will correct output to be consistent with numpy.percentile.

    Returns
    -------

    numpy.array

        computed quantiles from weighted distribution

    """
    values = np.array(values)
    quantiles = np.array(quantiles)

    if sample_weight is None:
        return np.percentile(values, quantiles)

    sample_weight = np.array(sample_weight)

    msg = 'quantiles should be in [0, 1]'
    assert np.all(quantiles >= 0) and np.all(quantiles <= 1), msg

    if not values_sorted:

        sorter = np.argsort(values)
        values = values[sorter]
        sample_weight = sample_weight[sorter]

    # Find the cumsum, and locate the center of each weight's mass
    # e.g. [0.25, 0.25, 0.5, 0] --> [0.125, 0.375, 0.75, 1]
    weighted_quantiles = np.cumsum(sample_weight) - 0.5 * sample_weight

    if old_style:

        # To be convenient with np.percentile
        # e.g. [0, 0.25, 0.25, 0.5, 0] --> [ 0, 0.2857, 0.7143, 1]
        weighted_quantiles -= weighted_quantiles[0]
        weighted_quantiles /= weighted_quantiles[-1]

    else:

        weighted_quantiles /= np.sum(sample_weight)

    result = np.interp(quantiles, weighted_quantiles, values)

    return result
